import { HighlightOptions } from 'highlight.run';
import { GraphQLClient } from 'graphql-request';
import * as Dom from 'graphql-request/dist/types.dom';
import LRU from 'lru-cache';
import * as http from 'http';

interface NodeOptions extends HighlightOptions {
    /**
     * Disables source code context lines for error reporting.
     * This may be useful for performance if your source
     * files are particularly large or memory is limited.
     * @default false
     */
    disableErrorSourceContext?: boolean;
    /**
     * Source files are cached in memory to speed up error reporting
     * and avoid costly disk access. The default cache size is 10MB,
     * but this can be overridden.
     * Specifying a value <= 0 removes all cache size limits.
     * @default 10
     */
    errorSourceContextCacheSizeMB?: number;
}

declare const HIGHLIGHT_REQUEST_HEADER = "x-highlight-request";
interface HighlightInterface {
    init: (options: NodeOptions) => void;
    isInitialized: () => boolean;
    consumeError: (error: Error, secureSessionId: string, requestId: string) => void;
    consumeEvent: (secureSessionId: string) => void;
    recordMetric: (secureSessionId: string, name: string, value: number, requestId?: string, tags?: {
        name: string;
        value: string;
    }[]) => void;
    flush: () => Promise<void>;
}
declare const H: HighlightInterface;

declare type Maybe<T> = T | null;
declare type InputMaybe<T> = Maybe<T>;
declare type Exact<T extends {
    [key: string]: unknown;
}> = {
    [K in keyof T]: T[K];
};
/** All built-in and custom scalars, mapped to their actual values */
declare type Scalars = {
    ID: string;
    String: string;
    Boolean: boolean;
    Int: number;
    Float: number;
    Any: any;
    Int64: any;
    Timestamp: any;
};
declare type BackendErrorObjectInput = {
    event: Scalars['String'];
    payload?: InputMaybe<Scalars['String']>;
    request_id: Scalars['String'];
    session_secure_id: Scalars['String'];
    source: Scalars['String'];
    stackTrace: Scalars['String'];
    timestamp: Scalars['Timestamp'];
    type: Scalars['String'];
    url: Scalars['String'];
};
declare type MetricTag = {
    name: Scalars['String'];
    value: Scalars['String'];
};
declare type MetricInput = {
    session_secure_id: Scalars['String'];
    group?: InputMaybe<Scalars['String']>;
    name: Scalars['String'];
    value: Scalars['Float'];
    category?: InputMaybe<Scalars['String']>;
    timestamp: Scalars['Timestamp'];
    tags?: InputMaybe<Array<MetricTag>>;
};
declare type PushBackendPayloadMutationVariables = Exact<{
    errors: Array<InputMaybe<BackendErrorObjectInput>> | InputMaybe<BackendErrorObjectInput>;
}>;
declare type PushBackendPayloadMutation = {
    __typename?: 'Mutation';
    pushBackendPayload?: any | null;
};
declare type MarkBackendSetupMutationVariables = Exact<{
    session_secure_id: Scalars['String'];
}>;
declare type MarkBackendSetupMutation = {
    __typename?: 'Mutation';
    markBackendSetup: string;
};
declare type PushMetricsMutationVariables = Exact<{
    metrics: Array<InputMaybe<MetricInput>> | InputMaybe<MetricInput>;
}>;
declare type PushMetricsMutation = {
    __typename?: 'Mutation';
    pushMetrics: string;
};
declare type SdkFunctionWrapper = <T>(action: (requestHeaders?: Record<string, string>) => Promise<T>, operationName: string, operationType?: string) => Promise<T>;
declare function getSdk(client: GraphQLClient, withWrapper?: SdkFunctionWrapper): {
    PushBackendPayload(variables: PushBackendPayloadMutationVariables, requestHeaders?: Dom.RequestInit['headers']): Promise<PushBackendPayloadMutation>;
    MarkBackendSetup(variables: MarkBackendSetupMutationVariables, requestHeaders?: Dom.RequestInit['headers']): Promise<MarkBackendSetupMutation>;
    PushMetrics(variables: PushMetricsMutationVariables, requestHeaders?: Dom.RequestInit['headers']): Promise<PushMetricsMutation>;
};
declare type Sdk = ReturnType<typeof getSdk>;

interface StackFrameContext {
    lineContent: string | undefined;
    linesBefore: string | undefined;
    linesAfter: string | undefined;
}
interface ErrorContextOptions {
    sourceContextCacheSizeMB?: number;
}
interface FileLines {
    lines: Map<number, string>;
    size: number;
}
declare class ErrorContext {
    _sourceContextCacheSizeMB: number | undefined;
    _cache: LRU<string, FileLines>;
    constructor(options: ErrorContextOptions);
    _getOrLoadLines(filename: string): Map<number, string>;
    getStackFrameContext(filename: string, lineNumber: number): StackFrameContext;
}

declare class Highlight {
    readonly FLUSH_TIMEOUT = 10;
    readonly BACKEND_SETUP_TIMEOUT: number;
    _graphqlSdk: Sdk;
    _backendUrl: string;
    _intervalFunction: ReturnType<typeof setInterval>;
    errors: Array<InputMaybe<BackendErrorObjectInput>>;
    metrics: Array<InputMaybe<MetricInput>>;
    lastBackendSetupEvent: number;
    _errorContext: ErrorContext | undefined;
    constructor(options: NodeOptions);
    recordMetric(secureSessionId: string, name: string, value: number, requestId?: string, tags?: {
        name: string;
        value: string;
    }[]): void;
    consumeCustomError(error: Error, secureSessionId: string, requestId: string): void;
    consumeCustomEvent(secureSessionId: string): void;
    flushErrors(): Promise<void>;
    flushMetrics(): Promise<void>;
    flush(): Promise<void>;
}

/** JSDoc */
interface MiddlewareError extends Error {
    status?: number | string;
    statusCode?: number | string;
    status_code?: number | string;
    output?: {
        statusCode?: number | string;
    };
}
/**
 * Express compatible error handler.
 * Exposed as `Handlers.errorHandler`
 */
declare function errorHandler(options?: NodeOptions): (error: MiddlewareError, req: http.IncomingMessage, res: http.ServerResponse, next: (error: MiddlewareError) => void) => void;

declare const handlers_errorHandler: typeof errorHandler;
declare namespace handlers {
  export {
    handlers_errorHandler as errorHandler,
  };
}

export { H, HIGHLIGHT_REQUEST_HEADER, handlers as Handlers, Highlight, NodeOptions };
